#
# spec file for package grpc
#
# Copyright (c) 2020 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define lver 8
%define lverp 1
%define src_install_dir /usr/src/%name
%{?!python_module:%define python_module() python-%{**} python3-%{**}}
Name:           grpc
Version:        1.25.0
Release:        lp152.2.3.1
Summary:        HTTP/2-based Remote Procedure Call implementation
License:        Apache-2.0
Group:          Development/Tools/Building
URL:            https://grpc.io/
Source:         https://github.com/grpc/grpc/archive/v%version.tar.gz
Source2:        %name-rpmlintrc
BuildRequires:  %{python_module Cython}
BuildRequires:  %{python_module devel}
BuildRequires:  %{python_module setuptools}
BuildRequires:  cmake
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  pkg-config
BuildRequires:  python-rpm-macros
BuildRequires:  zypper
BuildRequires:  pkgconfig(libcares)
BuildRequires:  pkgconfig(openssl) >= 1.0.1
BuildRequires:  pkgconfig(protobuf) >= 3.8.0
BuildRequires:  pkgconfig(zlib)

%description
The reference implementation of the gRPC protocol, done on top of
HTTP/2 with support for synchronous and asynchronous calls. gRPC uses
Protocol Buffers as the Interface Definition Language by default.

%package -n libgrpc%lver
Summary:        HTTP/2-based Remote Procedure Call implementation
Group:          System/Libraries
%if "%lver" == "7"
# prior error in packaging
Conflicts:      libgrpc6
%endif

%description -n libgrpc%lver
The reference implementation of the gRPC protocol, done on top of
HTTP/2 with support for synchronous and asynchronous calls. gRPC uses
Protocol Buffers as the Interface Definition Language by default.

%package -n libgrpc++%lverp
Summary:        HTTP/2-based Remote Procedure Call implementation
Group:          System/Libraries
%if "%lverp" == "1"
# prior error in packaging
Conflicts:      libgrpc6
%endif

%description -n libgrpc++%lverp
The reference implementation of the gRPC protocol, done on top of
HTTP/2 with support for synchronous and asynchronous calls. gRPC uses
Protocol Buffers as the Interface Definition Language by default.

%package devel
Summary:        Development files for grpc, a HTTP/2 Remote Procedure Call implementation
Group:          Development/Tools/Building
Requires:       libgrpc%lver = %version
Requires:       libgrpc++%lverp = %version

%description devel
This subpackage contains libraries and header files for developing
applications that want to make use of the gRPC reference implementation.

%package source
Summary:        Source code of gRPC
Group:          Development/Sources
BuildArch:      noarch

%description -n grpc-source
This subpackage contains source code of the gRPC reference implementation.

%package -n python2-grpcio
Summary:        Python language bindings for grpc, a HTTP/2 Remote Procedure Call implementation
Group:          Development/Libraries/Python
Requires:       libgrpc%lver = %version-%release
Requires:       python = %python2_version

%description -n python2-grpcio
This subpackage contains the python2 bindings.

%package -n python3-grpcio
Summary:        Python language bindings for grpc, a HTTP/2 Remote Procedure Call implementation
Group:          Development/Libraries/Python
Requires:       libgrpc%lver = %version-%release
Requires:       python = %python3_version

%description -n python3-grpcio
This subpackage contains the python3 bindings.

%prep
%autosetup -p1

%build
%define _lto_cflags %nil
# protoc is invoked strangely; make it happy with this dir or it will assert()
mkdir -p third_party/protobuf/src

export CFLAGS="%optflags -Wno-error"
export CXXFLAGS="$CFLAGS"
pushd .
%define __builddir cmake-build
%cmake -DgRPC_INSTALL=ON                  \
       -DgRPC_INSTALL_LIBDIR:PATH="%_lib" \
       -DgRPC_INSTALL_CMAKEDIR:PATH="%_libdir/cmake/grpc" \
       -DgRPC_CARES_PROVIDER=package      \
       -DgRPC_PROTOBUF_PROVIDER=package   \
       -DgRPC_SSL_PROVIDER=package        \
       -DZLIB_LIBRARY=%{_libdir}/libz.so  \
       -DgRPC_ZLIB_PROVIDER=package
export LD_LIBRARY_PATH="$PWD"
# generate gRPC*.cmake
%cmake_build
find . -name "*.cmake"
unset LD_LIBRARY_PATH
popd

# normal build
make %{?_smp_mflags} STRIP=/bin/true V=1 VERBOSE=1

# build python module
export GRPC_PYTHON_BUILD_WITH_CYTHON=True
export GRPC_PYTHON_BUILD_SYSTEM_OPENSSL=True
export GRPC_PYTHON_BUILD_SYSTEM_ZLIB=True
export GRPC_PYTHON_BUILD_SYSTEM_CARES=True
%python_build

%install
b="%buildroot"

# work around "Argument list too long"
ln -s "%buildroot" "b"
%make_install DESTDIR="b" prefix="b/%_prefix" STRIP=/bin/true V=1 VERBOSE=1

find "$b/%_includedir" -type f -exec chmod a-x {} +
pushd "$b/usr"
rm -fv lib/*.a share/grpc/*.pem
perl -i -pe 's{^prefix=.*}{prefix=%_prefix}' lib/pkgconfig/*.pc
perl -i -pe 's{^libdir=.*}{libdir=%_libdir}' lib/pkgconfig/*.pc
if test ! -d lib64 && test "%_lib" = lib64; then
	mv lib lib64
fi
popd

mkdir -p "$b/%_libdir/cmake"
cp -av cmake-build/gRPC*.cmake cmake-build/CMakeFiles/Export/*/*/cmake/grpc/gRPC*.cmake "$b/%_libdir/cmake"
rm -rf cmake-build

%python_install

# Install sources
make clean
rm -f "b" "a.out"
find . -type f "(" -name "*.so" -o -name "*.o" -o -name ".git*" ")" -exec rm -rf {} +
mkdir -p "%buildroot/%src_install_dir"
cp -r * "%buildroot/%src_install_dir"

%fdupes %buildroot/%_prefix

%post   -n libgrpc%lver -p /sbin/ldconfig
%postun -n libgrpc%lver -p /sbin/ldconfig
%post   -n libgrpc++%lverp -p /sbin/ldconfig
%postun -n libgrpc++%lverp -p /sbin/ldconfig

%files -n libgrpc%lver
%_libdir/libaddress_sorting.so.%{lver}*
%_libdir/libgpr*.so.%{lver}*
%_libdir/libgrpc*.so.%{lver}*

%files -n libgrpc++%lverp
%_libdir/libgrpc++*.so.%{lverp}*
%_libdir/libgrpcpp_channelz.so.%{lverp}*

%files devel
%license LICENSE
%_bindir/*
%_includedir/*
%_libdir/cmake/
%_libdir/pkgconfig/*.pc
%_libdir/*.so

%files source
%src_install_dir

%files -n python2-grpcio
%python2_sitearch/grpc*

%files -n python3-grpcio
%python3_sitearch/grpc*

%changelog
* Thu May 27 2021 Jan Engelhardt <jengelh@inai.de>
- Build and install .cmake files [boo#1184116]
* Thu Jan  9 2020 Michał Rostecki <mrostecki@opensuse.org>
- Remove patch for Bazel files, which is not needed anymore:
  * 0001-bazel-Replace-boringssl-with-openssl.patch
* Mon Dec 16 2019 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.25
  * Change RR to consider a subchannel in TF if it has failed
    since it was READY.
  * Do not create streams after a GOAWAY has been received.
  * Prevent HTTP2 parser from queueing a lot of induced frames.
- Drop gettid.patch
* Mon Nov 11 2019 Michał Rostecki <mrostecki@opensuse.org>
- Define BoringSSL as a Bazel dependency in WORKSPACE instead of
  grpc_deps.bzl - using new_local_repository function is now always
  allowed in imported modules.
* Tue Nov  5 2019 Michał Rostecki <mrostecki@opensuse.org>
- Add patch which replaces BoringSSL with OpenSSL as the SSL
  library when grpc is used as a Bazel dependency:
  * 0001-bazel-Replace-boringssl-with-openssl.patch
* Thu Oct 10 2019 Michał Rostecki <mrostecki@opensuse.org>
- Make sure that patches are included in the source package.
- Filter out several rpmlintrc warnings for the source package.
* Wed Oct  2 2019 Jan Engelhardt <jengelh@inai.de>
- Update to release 1.23.1
  * Unspecific refinements, improvements, and bug fixes.
- Add gettid.patch
- Split libgrpc into libgrpc++ because that has a SO version
  group of its own.
- Bump libgrpc6 to libgrpc7 as there is actually .so.7 files.
  Add suitable Conflicts markers.
* Thu Aug 22 2019 Jan Engelhardt <jengelh@inai.de>
- Disable LTO because it's broken.
* Sun Aug 18 2019 Jan Engelhardt <jengelh@inai.de>
- Update to new upstream release 1.23 [boo#1144068]
  * Fixed a bug where POST_RECV_MESSAGE was not being triggered.
  * Renamed root certificate bundle in gRPC-C++ pod.
  * Increased lower bound on DNS re-resolution period to 30 seconds.
  * Fixed a segfault when forking while using gevent.
  * Added channel idleness support
  * Added threadpool support
* Wed Jun  5 2019 Jan Engelhardt <jengelh@inai.de>
- Update to new upstream release 1.21.3
  * Add SPIFFE security stack to gRPC.
  * Re-enable ALPN check in gRPC C core SSL stack.
* Fri Apr 12 2019 sbrabec@suse.com
- Fix typo in Groups.
* Sat Mar 16 2019 Jan Engelhardt <jengelh@inai.de>
- Update to new upstream release 1.19.1
  * Use 32-bit nanopb fields.
  * Avoid SRV and TXT lookups for localhost.
  * Set c-ares as the default resolver.
* Mon Jan  7 2019 Michał Rostecki <mrostecki@suse.de>
- Add grpc-source package - some Go programs using gRPC need
  protobuf definitions which are included inside the source code,
  but are not included in the devel package.
* Sun Nov 11 2018 Jan Engelhardt <jengelh@inai.de>
- Update to new upstream release 1.16
  * Keepalive watchdog firing should return status UNAVAILABLE.
  * Set TCP_USER_TIMEOUT socket option for Linux.
  * Turn loading system root certificate as default.
  * Change pick_first to not unref unselected subchannels.
  * cmake: prevent C core from depending on libstdc++.
  * Implement Watch method in health check service.
* Sun Nov 11 2018 Thomas Bechtold <tbechtold@suse.com>
- update to 1.15.1:
  * Document SSL portability and performance considerations. See
    https://github.com/grpc/grpc/blob/master/doc/ssl-performance.md .
  * Simplify call arena size growth.
  * PF: Check connectivity state before watching.
  * Added system roots feature to load roots from OS trust store.
  * Fix re-resolution in pick first.
  * Allow error strings in final_info to propagate to filters on
    call destruction.
  * Add resolver executor.
  * Data race fix for lockfree_event.
  * Channelz: Expose new Core API.
  * cmake: disable assembly optimizations only when necessary.
  * C++ sync server: Return status RESOURCE_EXHAUSTED if no
    thread quota available.
  * Use correct target name for gflags-config.cmake.
  * Make should generate pkg-config file for gpr as well.
  * Restrict the number of threads in C++ sync server.
  * Allow reset of connection backoff.
- update to 1.14.2:
  * Add TROUBLESHOOTING.md guide
  * Fixed gRPC channels blocking indefinitely and not respecting
    deadlines on network disconnect.
  * Add user guide for keepalive.
  * Fix GRPC_ARG_HTTP2_MIN_RECV_PING_INTERVAL_WITHOUT_DATA_MS doc
    to restrict usage to server.
  * Fixed ordering in adding pending picks to Round Robin LB.
  * Implement local channel/server credentials for UDS connections.
  * Allow extra copy in zero-copy protector integrity-only mode.
  * Update to use the canonical version of LB proto.
  * Prefer using https_proxy over http_proxy.
  * Refactor to put c-ares queries under a combiner.
  * Remove already_closed parameter from fd_orphan.
  * Add channel arg to enable/disable http proxy.
  * Fix shutdown of closed fd when c-ares opens a second fd.
  * Fixed segmentation fault when product name from BIOS is empty.
  * Introduce server-side load reporting service.
  * Reorder steps in ServerBuilder::BuildAndStart().
  * Support Per-Method Codegen Generic on Server Side.
  * Protobuf-related headers are available in a subspec of
    gRPC-C++ podspec.
- update to 1.13.1:
  * gRPC stats will only be collected for debug builds or if
    GRPC_COLLECT_STATS is defined. It will be disabled for opt
    builds.
  * Fix for Issue #13553. Unlimited can now be set as the max
    receive message length.
  * cmake install now also installs should also installs the
    roots.pem file.
- Drop 0001-Allow-building-the-python-module-with-system-openssl.patch
  Applied upstream
- Drop 0002-Allow-building-the-python-module-with-system-zlib.patch
  Applied upstream
- 0003-Allow-building-the-python-module-with-system-cares.patch
  Applied upstream
* Sat Jun 23 2018 jengelh@inai.de
- Throw out silly -Werror in release builds.
* Fri May 25 2018 tbechtold@suse.com
- build grpc python module for python2 and python3
  * add 0001-Allow-building-the-python-module-with-system-openssl.patch
  * add 0002-Allow-building-the-python-module-with-system-zlib.patch
  * add 0003-Allow-building-the-python-module-with-system-cares.patch
* Fri May 18 2018 jengelh@inai.de
- Switch to plain makefiles since they generate more shared libs.
- Remove grpc-build.diff, grpc-versioning.diff.
* Fri May 18 2018 tbechtold@suse.com
- update to 1.12.0:
  Core:
  * Update protobuf submodule to 3.5.2 (#15114)
  * Make GrpcBuffer{Reader|Writer} Public (#14541)
  * Compile Out Tracers Option (#15108)
  * Fix bug in an address sorting comparison (#15208)
  * Fix: Check retry timer before starting resolving (#15156)
  * Fix c-ares resolution with default authority (#15073)
  * Fix c-ares resolver crash when DNS server unreachable (#15134)
  * Performance: Avoid low severity log message construction (#14945)
  * Peformance: Enable SIO_LOOPBACK_FAST_PATH on Windows (#14905)
  C#:
  * Add RpcException.StatusCode property (#15140)
  * Improvements to C# build instructions (#15178)
  Python:
  * Eliminate grpcio package dependency on protobuf package (#15034).
    Please pay extra attention if you are using gRPC with Protocol Buffers and
    your deployment scripts do not explicitly list the correct dependencies
    and used to rely on the transitive Python package dependency from grpcio to protobuf.
  * A new grpc.Channel.close method is introduced and correct use of gRPC
    Python now requires that channels be closed after use. No existing code
    should observably break when upgrading to 1.12 but code that creates
    channels but does not close them will not be supported in 1.13 and later.
  Core:
  * Fix: Check retry timer before starting resolving (#15156)
  * Fix c-ares resolution with default authority (#15073)
  * Fix c-ares resolver crash when DNS server unreachable (#15134)
  * Add core underpinnings for TLS session ticket support (#14483)
  * Experimental support for configurable retries (use at your own risk) (#10684)
  * Fail calls with wait_for_ready=false on transient resolver failure (#14733)
  * Fix a call stack ref flake (#14681)
  * c-ares DNS resolver sorts resolved addresses per RFC 6724 (#13290)
  * Use socklen_t instead of size_t for address length (fixes use case for some
    big endian systems) (#14464)
  * Initial part of channel tracing implementation (#13883, #14746, #14751)
  * Avoid crashes on environments with hot-plugged CPUs (#14712)
  * When building from source, Makefile now checks for minimum protobuf
    version 3.5.0 instead 3.0.0 (#14821)
  * gRPC Core public header no longer includes inttypes.h (#14676)
  * CMake improvements:
  * Add well-commented examples and continuous test for various ways of
    building gRPC as a dependency (via ExternalProject_Add in #14822,
    via add_subdirectory in #14864)
  * Remove internal tools from binaries installed by cmake install (#14850)
  * Simplify Android build example (#14885)
  * Enable SO_REUSEPORT in IPv6-only environments as well (#14748)
  C#:
  * Improvements in experimental support for Unity - added useful resources
    and improved support for loading grpc_csharp_ext on Unity (#14576, #14605)
  * Further prevent name collisions by using only fully qualified type
    names in C# code generator (#14524)
  * Fix C# connectivity watcher shutdown race (#14776)
  Objective-C:
  * Support for Keep-Alive (#14781)
  * Experimental C++ podspec now supports Apple framework (#14480)
  * Allow discarding generated implementation (leaving only protocol)
    with macro (#14490)
  * Better handling of network error (#14554)
  PHP:
  * Add missing -O2 compilation flag for pecl (#14768)
  * Update the reserved variable list to check in PHP code generator to
    match protobuf 3.5.0 (#13580)
  Python:
  * Progress towards full Beta API elimination (Beta API is and has been
    unsupported and will go away completely):
    Remove _face_interface_test tests for the Beta API (#14702)
  * Remove Python background poller thread (#14448)
    Note there is a small change in behavior. Before, clients would
    detect disconnected channels immediately.
  * Initial gevent compatibility (EXPERIMENTAL) (#14561)
  * Release the GIL when calling grpc_call_start_batch to improve
    performance under multithreaded scenarios. (#14801)
  Ruby:
  * Correct handling of underscores in package names by Protocol Buffers code
    generator for Ruby (#13634)
  * Source code and locally compiled (not cross-compiled) shared libraries
    (grpc_c.so files) are removed from the linux binary packages. This includes
    the packages targeting the x86_64-linux and x86-linux platforms (#14634)
  * Bug fix for a use-after-free related to access to a garbage collected
    object (#14809)
  * Get rid of the -Wno-format argument to compilation of the ruby extension
  * Documentation improvements (#14535)
  * Don’t raise GRPC::Core::CallErrors for failed reads or writes of bidi
    streaming calls (#14922)
  * Fix a race condition in an interrupt handler that could cause an
    assertion failure (#14953)
* Wed Mar 28 2018 jengelh@inai.de
- Update to new upstream release 1.10.0
  * Several features of core have been removed from the surface
    or GPR API as they were not used in any wrapped language
    (except C++) or in external applications.
  * Changed resolver and LB policy APIs to C++
  * C++ headers are moved from <grpc++/> to <grpcpp/>.
* Mon Feb  5 2018 jengelh@inai.de
- Update to new upstream release 1.9.0
  * LB policies request re-resolution without shutting down
  * On server, include receiving HTTP/2 settings in handshake
    timeout.
  * Add thread naming support on platforms that support it.
* Tue Jan 30 2018 jengelh@inai.de
- Update to new upstream release 1.8.6
  * Fix server segfault on invalid HTTP/2
* Fri Jan 19 2018 jengelh@inai.de
- Update to new upstream release 1.8.5
  * Fix a couple of memory leak issues in the PHP parts
* Tue Jan 16 2018 jengelh@inai.de
- Update to new upstream release 1.8.4
  * Fix HTTP/2 PING issues
  * Fix issues with poll engine
* Tue Jan  2 2018 crrodriguez@opensuse.org
- version 1.8.3
  * Eliminate superfluous log error messages
* Thu Dec 21 2017 jengelh@inai.de
- Update to new upstream release 1.8.2
  * Several LB policies fixes (grpclb and pick_first)
* Mon Dec 11 2017 jengelh@inai.de
- Update to new upstream release 1.7.3
  * Bugfixes in Load Balancing policies
  * Add flow-control and bugfixes to in-process transport
  * (C++) Async API extensions to improve multithreading support
    and reduce the need for explicit synchronization
  * (C++) Unary call API for generic (non-protobuf) RPCs
- Drop grpc-zlib.diff, grpc-protobuf.diff (fix included upstream)
- Add grpc-build.diff
* Wed Apr 19 2017 jengelh@inai.de
- Initial package (version 1.2.3) for build.opensuse.org
